
// SignSearchAmazon.java
// Andrew Davison, May 2009, ad@fivedots.coe.psu.ac.th

/* Use Amazon's Product Advertising API to look for a book cover image for 
  the title specified on the command line by the user.
  e.g:
      run SignSearchAmazon "Killer Game Programming in Java"
      
  At most the first 10 matches are listed (description and URL), and the first is 
  automatically saved as a PNG file called <fnm>Cover.png, where 
  fnm is the filename in the image's URL.

  Details on the Product Advertising API are in the developer guide at:
      http://docs.amazonwebservices.com/AWSECommerceService/latest/DG/

  This version of SearchAmazon signs the requests it sends to Amazon.
*/

import java.net.*;
import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import javax.xml.xpath.*;



public class SignSearchAmazon 
{
  // CHANGE THESE 2 LINES ************
  private static final String KEY_ID = "?????"; 
  private static final String SECRET_KEY = "?????;

  private static XPath xpath;
  private static SignedRequestsHelper signer;


  public static void main(String args[])
  {       
    if (args.length == 0) {
      System.err.println("Supply a title");
      System.exit(1);
    }

    xpath = createXPath();
    signer = new SignedRequestsHelper(KEY_ID, SECRET_KEY);

    // get the Amazon ASIN for a matching book title
    String asin = bookTitleSearch(args[0]);    // search phase

    // get the URL of the image used with that ASIN listing
    String urlStr = imageLookup(asin);   // lookup phase

    WebUtils.downloadImage(urlStr);
  }  // end of main()



  private static XPath createXPath()
  // create a XPath object that understands the Amazon Namespace
  {
    XPath xpath = XPathFactory.newInstance().newXPath();
    NamespaceContextImpl ns = new NamespaceContextImpl("amz",
            "http://webservices.amazon.com/AWSECommerceService/2005-10-05");
    xpath.setNamespaceContext(ns);

    return xpath;
  }  // end of createXPath()



  private static String bookTitleSearch(String title)
  // query Amazon for a book with the specified title, and return its ASIN
  {
    System.out.println("Searching Amazon for the book title \"" + title + "\"");
    try {
      String ueTitle = URLEncoder.encode(title, "UTF-8");

      HashMap<String, String> urlParams = new HashMap<String, String>();
      urlParams.put("Service", "AWSECommerceService");
      urlParams.put("Operation", "ItemSearch");         // use ItemSearch
      urlParams.put("SearchIndex", "Books");            // limit the search to books
      urlParams.put("Power", "title:\"" + title + "\"");   // double quotes around title

      String urlStr = signer.sign(urlParams);

      Document doc = WebUtils.parse( WebUtils.webGet(urlStr) );
      // WebUtils.saveDoc(doc, "temp.xml");

      return getASIN(doc);
    }
	catch (Exception e) 
	{ System.out.println(e); 
      System.exit(1);
    }

    return null;  
  }  // end of bookTitleSearch()



  private static String getASIN(Document doc)
  // print matching titles and ASINs, and return the first ASIN
  {
    try {
      String totalStr =      // get the total no. of results
         (String) xpath.evaluate("//amz:TotalResults", doc, XPathConstants.STRING);
      System.out.println("Total no. of item matches: " + totalStr);

      // get the titles and their ASINs
      NodeList nodes = doc.getElementsByTagName("Item"); // get all the items
      String firstAsin = null; 
      for (int i = 0; i < nodes.getLength(); i++) {     // at most 10 by default
        System.out.print( (i+1) + ". ");
        Element elem = (Element) nodes.item(i);
        System.out.println("Title: " + WebUtils.getElemText(elem, "Title"));
        String asin = WebUtils.getElemText(elem, "ASIN");  // get the ASIN for this item
        System.out.println("   ASIN: " + asin);
        if (i == 0)  // store first match
          firstAsin = asin;
      }
      return firstAsin;
    }
    catch(Exception e)
    { System.out.println(e);  
      System.exit(1);
    }
    return null;
   }  // end of getASIN()


  private static String imageLookup(String asin)
  {
    System.out.println("Retrieving large image for Amazon item with ASIN: " + asin);
    try {
      HashMap<String, String> urlParams = new HashMap<String, String>();
      urlParams.put("Service", "AWSECommerceService");
      urlParams.put("Operation", "ItemLookup");   // use ItemLookup
      urlParams.put("ItemId", asin);              // use the asin to get the item
      urlParams.put("ResponseGroup", "Images");   // limit the response to images
             // ACCESS_KEY_ID and timestamp added by signer code

      String urlStr = signer.sign(urlParams);

      Document doc = WebUtils.parse( WebUtils.webGet(urlStr) );
      WebUtils.saveDoc(doc, "temp.xml");

      // return the URL of the large image for this item
      return (String) xpath.evaluate("//amz:Item/amz:LargeImage/amz:URL", 
                                                    doc, XPathConstants.STRING);
    }
	catch (Exception e) 
	{ System.out.println( e ); 
      System.exit(1);
    }

    return null;  
  }  // end of imageLookup()


 } // end of SignSearchAmazon class
