
// SignedRequestsHelper.java
// from Amazon, May 2009
// Slightly modified by Andrew Davison, May 2009, ad@fivedots.coe.psu.ac.th

/* Java Sample Code for Calculating Signature Version 2 Signatures
   From the DG:
          Requests > Request Authentication > Authenticating REST Requests > 
                 Java Sample Code for Calculating Signature Version 2 Signatures

  The following Java code sample shows how to calculate a signature (version 2). 
  This sample has code for creating the canonical string, for base64 encoding, 
  and for HMAC encoding for generating the signature.

  From Product Advertising API developer guide:
  http://docs.amazonwebservices.com/AWSECommerceService/latest/DG/

  The steps carried out here are illustrated in the DG section:
      "Requests > Request Authentication > 
             Authenticating REST Requests > Example REST Requests"

  There are 10 steps, which are labelled in the code below.

  Changes:
     - pass in ID and key in the constructor
     - added some exception checking in SignedRequestsHelper
     - added comments
*/


import java.io.*;
import java.net.*;
import java.security.*;
import java.text.*;
import java.util.*;
import javax.crypto.*;
import javax.crypto.spec.*;

import org.apache.commons.codec.binary.Base64;


public class SignedRequestsHelper
{
  private static final String UTF8_CHARSET = "UTF-8";
  private static final String HMAC_SHA256_ALGORITHM = "HmacSHA256";
  private static final String REQUEST_URI = "/onca/xml";
  private static final String REQUEST_METHOD = "GET";

  private String endpoint = "ecs.amazonaws.com"; // must be lowercase

  private String awsAccessKeyId, awsSecretKey;

  private SecretKeySpec secretKeySpec = null;
  private Mac mac = null;



  public SignedRequestsHelper(String keyId, String secretKey)
  {
    awsAccessKeyId = keyId;
    awsSecretKey = secretKey;
    try {
      /* initialise an RFC 2104-compliant HMAC with the 
         SHA256 hash algorithm (part of step 8)  */
      byte[] secretyKeyBytes = awsSecretKey.getBytes(UTF8_CHARSET);
      secretKeySpec = new SecretKeySpec(secretyKeyBytes, HMAC_SHA256_ALGORITHM);
      mac = Mac.getInstance(HMAC_SHA256_ALGORITHM);
      mac.init(secretKeySpec);
    }
    catch(Exception e)
    {  System.out.println(e);  }
  }  // end of SignedRequestsHelper()



  public String sign(Map<String, String> params)
  /* input params have already been split into key/value pairs, as
     required by step 3. Returns the signed URL
  */
  {
    params.put("AWSAccessKeyId", awsAccessKeyId);
    params.put("Timestamp", timestamp());    // step 1. Enter the timestamp

    SortedMap<String, String> sortedParamMap = new TreeMap<String, String>(params);   
              // step 4: sort the key/value pairs by byte value

    String canonicalQS = canonicalize(sortedParamMap);      // steps 5 and 2

    String toSign = REQUEST_METHOD + "\n" + endpoint + "\n" + 
                    REQUEST_URI + "\n" + canonicalQS;
       // step 6. Prepend three request lines to the message
       // toSign is illustrated by step 7

    String hmac = hmac(toSign);                 // step 8
    String sig = percentEncodeRfc3986(hmac);    // step 9
    String url = "http://" + endpoint + REQUEST_URI + "?" + 
                            canonicalQS + "&Signature=" + sig;
        // step 10: Add the URL encoded signature to the request
    return url;
  }  // end of sign()



  private String hmac(String stringToSign)
  /* step 8. Calculate an RFC 2104-compliant HMAC with the 
     SHA256 hash algorithm  */
  {
    String signature = null;
    try {
      byte[] data = stringToSign.getBytes(UTF8_CHARSET);
      byte[] rawHmac = mac.doFinal(data);
      Base64 encoder = new Base64();
      signature = new String(encoder.encode(rawHmac));
    }
    catch (UnsupportedEncodingException e) {
      throw new RuntimeException(UTF8_CHARSET + " is unsupported!", e);
    }
    return signature;
  }  // end of hmac()



  private String timestamp()
  // return the timestamp, e.g. 2009-01-01T12:00:00Z
  {
    String timestamp = null;
    Calendar cal = Calendar.getInstance();
    DateFormat dfm = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

    dfm.setTimeZone(TimeZone.getTimeZone("GMT"));
    timestamp = dfm.format(cal.getTime());
    return timestamp;
  }  // end of timestamp()



  private String canonicalize(SortedMap<String, String> sortedParamMap)
  /* Step 5. Rejoin the sorted key/value map with ampersands. 
     The result is a 'canonical' string. */
  {
    if (sortedParamMap.isEmpty())
      return "";

    StringBuffer buffer = new StringBuffer();
    Iterator<Map.Entry<String, String>> iter = sortedParamMap.entrySet().iterator();

    while (iter.hasNext()) {
      Map.Entry<String, String> kvpair = iter.next();
      buffer.append( percentEncodeRfc3986(kvpair.getKey()) );   // step 2
      buffer.append("=");
      buffer.append( percentEncodeRfc3986(kvpair.getValue()) );  // step 2
            // step 2: URL encode the request's comma (,) and colon (;) chars
      if (iter.hasNext())
        buffer.append("&");
    }
    String cannoical = buffer.toString();
    return cannoical;
  }  // end of canonicalize()



  private String percentEncodeRfc3986(String s)
  // step 9: URL encode plus (+) and equal (=) chars in signature
  {
    String out;
    try {
      out = URLEncoder.encode(s, UTF8_CHARSET).replace("+", "%20")
                           .replace("*", "%2A").replace("%7E", "~");
    }
    catch (UnsupportedEncodingException e) 
    {  out = s;  }
    return out;
  }  // end of percentEncodeRfc3986()


}  // end of SignedRequestsHelper class
