
// SearchJYahoo.java
// Andrew Davison, May 2009, ad@fivedots.coe.psu.ac.th

/* Use Yahoo's Product Search API to look for a book cover image for 
  the title specified on the command line by the user.
  e.g:
      UseProxy SearchJYahoo "Killer Game Programming in Java"
      
  At most the first 5 matches are listed (title and picture URL), and the first 
  picture URL is downloaded and saved as a PNG file called <fnm>Cover.png, where 
  fnm is the filename in the image's URL.

  Details on the Product Search API can be found at:
       http://developer.yahoo.com/shopping/V3/productSearch.html

  This example differs from SearchYahoo in that the Product Search query
  returns a JSON structure which is converted to a HashMap using 
  the Jackson Java JSON-processor (http://jackson.codehaus.org/) and 
  scanned using JXPath (http://commons.apache.org/jxpath/), a JSON version
  of XPath.
*/

import java.net.*;
import java.io.*;
import java.util.*;

import org.codehaus.jackson.*;
import org.codehaus.jackson.map.*;
import org.apache.commons.jxpath.*;


public class SearchJYahoo 
{
  // **** CHANGE THIS LINE ****
  private static String APP_ID = "?????"; 


  public static void main(String args[])
  {       
    if (args.length == 0) {
      System.err.println("Supply a title");
      System.exit(1);
    }
    String imUrlStr = bookTitleSearch(args[0]);
    WebUtils.downloadImage(imUrlStr);
  }  // end of main()



  private static String bookTitleSearch(String title)
  // query Yahoo for a book with the specified title, and return its image URL
  {
    System.out.println("Searching Yahoo for the book title \"" + title + "\"");
    try {
      String ueTitle = URLEncoder.encode(title, "UTF-8");

      String urlStr = "http://shopping.yahooapis.com/ShoppingService/V3/productSearch?" +
                          "appid=" + APP_ID + "&" + 
                          "department=56&" +     // book department
                          "query=" + ueTitle + "&" +
                          "results=5&" +
                          "output=json" ;  // JSON output

      String jsonStr = WebUtils.webGetString(urlStr);

      ObjectMapper mapper = new ObjectMapper();
      // Class type = (HashMap<String,Object>).class;
      // HashMap<String,Object> jsonMap = mapper.readValue(jsonStr, type);
      HashMap<String,Object> jsonMap = mapper.readValue(jsonStr, HashMap.class);
      saveJSON("temp.json", mapper, jsonStr);

      return getPictureURL(jsonMap);
    }
	catch (Exception e) 
	{ System.out.println(e); 
      System.exit(1);
    }

    return null;  
  }  // end of bookTitleSearch()



  private static void saveJSON(String fnm, ObjectMapper mapper, String jsonStr)
  // save a formatted version of the JSON string to fnm
  {
    SerializationConfig config = mapper.getSerializationConfig();
    config.set(SerializationConfig.Feature.INDENT_OUTPUT, true);  // pretty print 

    try {
      JsonNode rootNode = mapper.readValue(jsonStr, JsonNode.class);
      mapper.writeValue(new File(fnm), rootNode);
      System.out.println("Saved JSON structure to " + fnm);
    }
    catch(IOException e)
    {  System.out.println("Could not save JSON structure to " + fnm);  }
  } // end of saveJSON()




  private static String getPictureURL(HashMap<String,Object> jsonMap)
  /* print matching titles and GridImage picture URLs, and 
     return the first picture URL */
  {
    try {
      JXPathContext context = JXPathContext.newContext(jsonMap);

      // get the total no. of results (the @totalResultsAvailable attribute)
      String totalStr =   
          (String) context.getValue("/Products/__ATTRIBUTES/totalResultsAvailable");
                              // "__ATTRIBUTES" is used in the HashMap transformation
      System.out.println("Total no. of products matches: " + totalStr);

      Iterator prodIter = context.iterate("//Product");   // all products
      JXPathContext prodContext;
      String firstURL = null;
      int i=0;
      while(prodIter.hasNext()){
        System.out.print( (i+1) + ". ");
        prodContext = JXPathContext.newContext( prodIter.next() );  // next product

        System.out.println("Title: " + prodContext.getValue("//ProductName") );
        String urlStr = null;
        try {  // Grid Image may not be present
          urlStr = (String) prodContext.getValue("//GridImage/Url");   
        }
        catch(JXPathNotFoundException e) {}
        System.out.println("   URL: " + urlStr);
        if (firstURL == null)  // store first URL match that isn't null
          firstURL = urlStr;
        i++;
      }
      return firstURL;
    }
    catch (Exception e) 
    { System.out.println(e); }
    
    return null;
  } // end of getPictureURL()


 } // end of SearchJYahoo class
