
// SearchEBay.java
// Andrew Davison, May 2009, ad@fivedots.coe.psu.ac.th

/* Use ebay's Shopping API to look for a book cover image for 
  the title specified on the command line by the user.
  e.g:
      UseProxy SearchEBay "Killer Game Programming in Java"
      
  At most the first 5 matches are listed (description and URL), and the first is 
  automatically saved as a PNG file called <fnm>Cover.png, where 
  fnm is the filename in the image's URL.

  Details on the Shopping API in Java can be found at:
       http://developer.ebay.com/developercenter/java/shopping/
*/

import java.net.*;
import java.io.*;

import org.w3c.dom.*;
import javax.xml.xpath.*;



public class SearchEBay 
{
  // **** CHANGE THIS LINE ****
  private static String APP_ID = "?????"; 

  private static XPath xpath;


  public static void main(String args[])
  {       
    if (args.length == 0) {
      System.err.println("Supply a title");
      System.exit(1);
    }

    xpath = createXPath();

    String itemID = bookTitleSearch(args[0]);  // search phase
    String imUrlStr = pictureLookup(itemID);   // lookup phase

    WebUtils.downloadImage(imUrlStr);
  }  // end of main()



  private static XPath createXPath()
  // create a XPath object that understands the ebay Namespace
  {
    XPath xpath = XPathFactory.newInstance().newXPath();
    NamespaceContextImpl ns = new NamespaceContextImpl("ebay",
                                        "urn:ebay:apis:eBLBaseComponents");
    xpath.setNamespaceContext(ns);

    return xpath;
  }  // end of createXPath()



  private static String bookTitleSearch(String title)
  // query ebay for a book with the specified title, and return its item ID
  {
    System.out.println("Searching eBay for the book with the keywords \"" + title + "\"");
    try {
      String ueTitle = URLEncoder.encode(title, "UTF-8");

      String urlStr = "http://open.api.ebay.com/shopping?" +
                          "callname=FindItemsAdvanced&" +   // using FindItemsAdvanced op.
                          "responseencoding=XML&" +   // could be JSON
                          "appid=" + APP_ID + "&" + 
                          "siteid=0&" +
                          "version=525&" +
                          "CategoryID=267&" +    // restrict to books
                          "QueryKeywords=" + ueTitle + "&" +
                          "MaxEntries=5" ;       // at most 5 matches returned

      Document doc = WebUtils.parse( WebUtils.webGet(urlStr) );
      WebUtils.saveDoc(doc, "temp.xml");

      return getItemID(doc);
    }
	catch (Exception e) 
	{ System.out.println(e); 
      System.exit(1);
    }

    return null;  
  }  // end of bookTitleSearch()



  private static String getItemID(Document doc)
  // print matching titles and item IDs, and return the first item ID
  {
    try {
      String totalStr =      // get the total no. of results
         (String) xpath.evaluate("//ebay:TotalItems", doc, XPathConstants.STRING);
      System.out.println("Total no. of item matches: " + totalStr);

      // get the titles and their item IDs
      NodeList nodes = doc.getElementsByTagName("Item"); // get all the items
      String firstItemID = null; 
      for (int i = 0; i < nodes.getLength(); i++) { 
        System.out.print( (i+1) + ". ");
        Element elem = (Element) nodes.item(i);
        System.out.println("Title: " + WebUtils.getElemText(elem, "Title"));
        String itemID = WebUtils.getElemText(elem, "ItemID");  // get the ID for this item
        System.out.println("   ItemID: " + itemID);
        if (i == 0)  // store first match
          firstItemID = itemID;
      }
      return firstItemID;
    }
    catch(Exception e)
    { System.out.println(e);  
      System.exit(1);
    }
    return null;
  }  // end of getItemID()



  private static String pictureLookup(String itemID)
  // get the details for the specified item ID and return it's picture URL
  {
    System.out.println("Retrieving picture for eBay item with ID: " + itemID);
    try {
      String urlStr = "http://open.api.ebay.com/shopping?" +
                          "callname=GetSingleItem&" +    // using GetSingleItem op.
                          "responseencoding=XML&" +   // could be JSON
                          "appid=" + APP_ID + "&" + 
                          "siteid=0&" +
                          "version=525&" +
                          "ItemID=" + itemID + "&" ;    // 

      Document doc = WebUtils.parse( WebUtils.webGet(urlStr) );
      WebUtils.saveDoc(doc, "temp.xml");

      return getPictureURL(doc);
    }
	catch (Exception e) 
	{ System.out.println( e ); 
      System.exit(1);
    }

    return null;  
  }  // end of pictureLookup()


  private static String getPictureURL(Document doc)
  // an item may have 0, 1, or many picture URLs; return the first one
  {
    try {
      NodeList nodes = (NodeList) xpath.evaluate("//ebay:PictureURL/text()", 
                                                         doc, XPathConstants.NODESET);
      if (nodes.getLength() == 0)
        return null;
      if (nodes.getLength() > 1)
        System.out.println("No. of pictures: " + nodes.getLength());

      return (String) nodes.item(0).getNodeValue();        
    }
    catch(Exception e)
    {  System.out.println(e);  }

    return null;
   }  // end of getPictureURL()


 } // end of SearchEBay class
